<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/abstract_preloaded_regular_screen.php';

///////////////////////////////////////////////////////////////////////////

$month_arr = array('01' => 'янв',
                   '02' => 'фев',
                   '03' => 'мар',
                   '04' => 'апр',
                   '05' => 'мая',
                   '06' => 'июн',
                   '07' => 'июл',
                   '08' => 'авг',
                   '09' => 'сен',
                   '10' => 'окт',
                   '11' => 'ноя',
                   '12' => 'дек'
                  );


class I24hPrivateScreen extends AbstractPreloadedRegularScreen
    implements UserInputHandler
{
    const ID = 'private';

    public static function get_media_url_str($pin)
    {
        return MediaURL::encode(
            array('screen_id' => self::ID, 'pin' => $pin));
    }

    public static function get_item_media_url_str($pin, $type, $service_id)
    {
        return MediaURL::encode(
            array(
                'screen_id' => self::ID,
                'type' => $type,
                'pin' => $pin,
                'service_id' => $service_id));
    }

    public function get_handler_id()
    {
        return self::ID;
    }

    /////////////////////////////////////////////////////////////////////// 

    private /* I24hPlugin */ $plugin;

    public function __construct(I24hPlugin $plugin)
    {
        parent::__construct(self::ID, I24hConfig::GET_PRIVATE_FOLDER_VIEWS());

        $this->plugin = $plugin;

        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

    public function get_action_map(MediaURL $media_url, &$plugin_cookies)
    {
        return array
        (
            GUI_EVENT_KEY_ENTER =>
                UserInputHandlerRegistry::create_action(
                    $this, 'handle_enter'),
            GUI_EVENT_KEY_INFO =>
                UserInputHandlerRegistry::create_action(
                    $this, 'handle_info'),
        );
    }

    private function info_dialog_action(
        $service_id, $selected_media_url,
        $title, $msg_lines, $apply_name)
    {
        $defs = array();

        foreach ($msg_lines as $line)
            ControlFactory::add_label($defs, null, $line);

        ControlFactory::add_vgap($defs, 30);

        $add_params = array(
            'service_id' => $service_id,
            'selected_media_url' => $selected_media_url);

        ControlFactory::add_close_dialog_and_apply_button($defs,
            $this, $add_params,
            $apply_name, 'OK', 200, true);

        return ActionFactory::show_dialog($title, $defs, true, 0,
            array('initial_sel_ndx' => 0));
    }

    
    private function get_yes_no_dialog_action(
        $service_id, $selected_media_url,
        $title, $msg_lines, $apply_name, $yes_by_default)
    {
        $defs = array();

        foreach ($msg_lines as $line)
            ControlFactory::add_label($defs, null, $line);

        ControlFactory::add_vgap($defs, 30);

        $add_params = array(
            'service_id' => $service_id,
            'selected_media_url' => $selected_media_url);

        ControlFactory::add_close_dialog_and_apply_button($defs,
            $this, $add_params,
            $apply_name, 'Да', 200, true);

        ControlFactory::add_vgap($defs, -3);

        ControlFactory::add_close_dialog_button($defs,
            'Нет', 200, true);

        $initial_sel_ndx = $yes_by_default ? 0 : 1;
        return ActionFactory::show_dialog($title, $defs, true, 0,
            array('initial_sel_ndx' => $initial_sel_ndx));
    }

    private function get_pay_dialog_action(
        $service_id, $selected_media_url,
        $title, $msg_lines, $apply_name, $yes_by_default, $ps_arr)
    {
        $defs = array();

        foreach ($msg_lines as $line)
            ControlFactory::add_label($defs, null, $line);

        ControlFactory::add_vgap($defs, 30);
        
        ControlFactory::add_combobox($defs, $this, null, 'paysystem', "Платежная система",
            0, $ps_arr, 0, false, false);
        

        $add_params = array(
            'service_id' => $service_id,
            'selected_media_url' => $selected_media_url);

        ControlFactory::add_close_dialog_and_apply_button($defs,
            $this, $add_params,
            $apply_name, 'Подписаться', 200, true);

        ControlFactory::add_vgap($defs, -3);

        ControlFactory::add_close_dialog_button($defs,
            'Отмена', 200, true);

        $initial_sel_ndx = $yes_by_default ? 0 : 1;
        return ActionFactory::show_dialog($title, $defs, true, 0,
            array('initial_sel_ndx' => $initial_sel_ndx));
    }

    private function do_edit_pin_action()
    {
        $defs = array();

        ControlFactory::add_text_field($defs, $this, null,
            'current_pin', 'Текущий PIN-код:',
            '', true, true, false, false, 500);

        ControlFactory::add_text_field($defs, $this, null,
            'new_pin', 'Новый PIN-код:',
            '', true, true, false, false, 500);

        ControlFactory::add_text_field($defs, $this, null,
            'new_pin_copy', 'Подтверждение:',
            '', true, true, false, false, 500);

        ControlFactory::add_vgap($defs, 50);

        ControlFactory::add_button($defs, $this, null,
            'apply_pin', null, 'Применить', 300, true);

        ControlFactory::add_vgap($defs, -3);

        ControlFactory::add_close_dialog_button($defs,
            'Отмена', 300, true);

        return ActionFactory::show_dialog('Смена PIN-кода', $defs, true);
    }

    private function do_reset_auth_settings_action()
    {
        $defs = array();

        ControlFactory::add_label($defs, null,
            'Вы уверены, что хотите сбросить настройки авторизации?');

        ControlFactory::add_vgap($defs, 30);
        
        ControlFactory::add_close_dialog_and_apply_button($defs, $this, null,
            'apply_reset_auth_settings', 'Да', 200, true);

        ControlFactory::add_vgap($defs, -3);

        ControlFactory::add_close_dialog_button($defs,
            'Нет', 200, true);

        return ActionFactory::show_dialog(
            'Подтверждение', $defs, true, 0,
            array('initial_sel_ndx' => 2));
    }

    private function apply_pin($user_input, $plugin_cookies)
    {
        if ($user_input->new_pin !== $user_input->new_pin_copy)
        {
            return ActionFactory::show_error(
                false, 'Ошибка', array('Новый PIN-код и его подтверждение не совпадают'));
        }

        $session = $this->plugin->get_session($plugin_cookies);
        $session->ensure_logged_in($plugin_cookies);
        
        $m = $session->getMemberByPin($user_input->current_pin);
        if (is_null($m))
        {
            return ActionFactory::show_error(
                false, 'Ошибка', array('Неверный текущий PIN-код'));
        }

        try
        {
            $session->editMember(
                $m->id, $user_input->new_pin,
                $m->name, $m->spendingLimit, $m->prLevel);
        }
        catch (I24hException $e)
        {
            $error_message = 'Обратитесь в тех-поддержку.';

            if ($e->getCode() === I24hException::OPERATION_FAILED)
            {
                // See SUIExceptionCodes.txt.
                switch ($e->getApiErrorCode())
                {
                    case 302:
                        $error_message = 'Неправильный формат PIN-кода.';
                        break;

                    case 600:
                        $error_message = 'PIN-код уже используется.';
                        break;
                }
            }

            return ActionFactory::show_error(
                false, 'Невозможно сменить PIN-код', array($error_message));
        }

        return ActionFactory::close_dialog();
    }

    private function apply_reset_auth_settings(&$plugin_cookies)
    {
        hd_print("Setup: applying reset auth settings");

        unset($plugin_cookies->user_name);
        unset($plugin_cookies->user_password);

        $this->plugin->close_session();

        return ActionFactory::show_error(
            true, 'Настройки авторизации сброшены.');
    }

    private function handle_reset_auth_user_input(
        &$user_input, &$plugin_cookies)
    {
        $media_url = MediaURL::decode($user_input->selected_media_url);

        return null;
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        hd_print('TvSubscriptions: handle_user_input:');
        foreach ($user_input as $key => $value)
            hd_print("  $key => $value");

        if (isset($user_input->control_id))
        {
            if ($user_input->control_id === 'apply_pin')
                return $this->apply_pin($user_input, $plugin_cookies);
            if ($user_input->control_id === 'apply_reset_auth_settings')
                return $this->apply_reset_auth_settings($plugin_cookies);
        }

        if (!isset($user_input->selected_media_url))
            return null;
        $media_url = MediaURL::decode($user_input->selected_media_url);
        
        if ($media_url->screen_id == 'setup') {
            return ActionFactory::open_folder();
        }

        if ($media_url->type == 'edit_pin')
            return $this->do_edit_pin_action();

        if ($media_url->type == 'reset_auth')
            return $this->do_reset_auth_settings_action();

        $session = $this->plugin->get_session($plugin_cookies);
        $session->ensure_logged_in($plugin_cookies);

        $srv = $session->getServiceById($media_url->service_id);

        try
        {
            $balance = -$session->getBalance();
        }
        catch (I24hException $e)
        {
            return ActionFactory::show_error(
                false,
                'Ошибка проверки баланса',
                array('Обратитесь в тех-поддержку.'));
        }

        if (is_null($srv))
            return null;

        if ($user_input->control_id == 'handle_enter')
        {
        	if (isset($srv->starts)) {
        		$start_time = strftime('%d.%m.%Y', date_timestamp_get(date_create($srv->starts)));
        		return ActionFactory::show_error(
        			false, 'Ошибка', array('Данный тариф уже запланирован с '.$start_time));
        	}
        	else 
            if (($srv->isSubscribed) && (!(isset($srv->finished))))
            {
                return $this->get_yes_no_dialog_action(
                    $media_url->service_id,
                    $user_input->selected_media_url,
                    'Подтверждение',
                    array
                    (
                        'Вы уверены, что хотите отписаться от пакета:',
                        '"' . $srv->name . '"?',
                    ),
                    'apply_unsubscribe',
                    false);
            }
            else
            {
            	$ps = $session->getPaysystemsList();
            	$ps_arr = array();
            	$has_ps = 0;
            	
            	foreach ($ps as $psystem) {
            	    if ($psystem->systype == 2) {
            	    	$ps_arr[$psystem->id] = $psystem->name;
            	    	$has_ps = 1;
            	    }
            	    //hd_print("$psystem->id -> $psystem->name");
            	}
            	//if (($has_ps) || ($media_url->service_id == 12)) {
            	    return $this->get_pay_dialog_action(
            	    	$media_url->service_id,
            	    	$user_input->selected_media_url,
            	    	'Подтверждение',
            	    	array
            	    	(
            	    	    'Подтвердите подписку на пакет',
            	    	    '"' . $srv->name . '"',
            	    	    'Текущий баланс: ' . $balance . ' руб',
            	    	    'Стоимость подписки: ' . $srv->price . ' руб. в месяц',
            	    	    ),
            	    	'apply_subscribe',
            	    	true,
            	    	$ps_arr);
            	/*} else {
            	    return ActionFactory::show_error(
            	    	false,
            	    	'Подписка',
            	    	array('Для оплаты пакета пожалуйста обратитесь к своему провайдеру.'));
            	}*/
            }
        }
        else if ($user_input->control_id == 'apply_subscribe')
        {
            try
            {
                $res = $session->addServiceToSubscription(
                    $user_input->service_id);
            }
            catch (I24hException $e)
            {
                return ActionFactory::show_error(
                    false,
                    'Невозможно добавить подписку',
                    array('Обратитесь в тех-поддержку.'));
            }

            if ($res->err == 0) {
            	$balance = $session->getBalance();
            	
                /*return ActionFactory::show_error(
                    false,
                    'Подписка успешно добавлена',
                    $balance > 0 ? 'Пожалуйста, пополните свой счет для начала действия тарифа' : array($res->msg));*/
            } else {
                return $this->info_dialog_action(
                    $media_url->service_id,
                    $user_input->selected_media_url,
                    'Информация',
                    array($res->msg),
                    'apply_reload'  );
                
               /* return ActionFactory::show_error(
                    false,
                    'Невозможно добавить подписку',
                    array($res->msg));*/
            }
            
            $this->plugin->get_tv()->unload_channels();
            

            $items = $this->get_all_folder_items(
                $media_url, $plugin_cookies);
            return ActionFactory::update_regular_folder(
                HD::create_regular_folder_range($items), true);
        }
        else if ($user_input->control_id == 'apply_unsubscribe')
        {
            try
            {
                $session->removeServiceFromSubscription(
                    $user_input->service_id);
            }
            catch (I24hException $e)
            {
                return ActionFactory::show_error(
                    false,
                    'Невозможно удалить подписку',
                    array('Обратитесь в тех-поддержку.')); 
            }

            $this->plugin->get_tv()->unload_channels();

            $items = $this->get_all_folder_items(
                $media_url, $plugin_cookies);
            return ActionFactory::update_regular_folder(
                HD::create_regular_folder_range($items), true);
        }
        else if ($user_input->control_id == 'apply_reload')
        {
            $this->plugin->get_tv()->unload_channels();

            $items = $this->get_all_folder_items(
                $media_url, $plugin_cookies);
            return ActionFactory::update_regular_folder(
                HD::create_regular_folder_range($items), true);
        }
        else if ($user_input->control_id == 'handle_info')
        {
            /*
            $chans = array();
            
            $chans[] = "Состав пакета ТВ-каналов \"$srv->name\"";
            
            foreach ($srv->channels as $chan)
            {
            	$chans[] = $chan->name;
            }
            
            return ActionFactory::show_popup_menu($chans, 0);*/
            
            $defs = array();

            if (!preg_match('/^\s*$/', $srv->description))
            {
                ControlFactory::add_label($defs,
                    null, $srv->description);

                ControlFactory::add_vgap($defs, 30);
            }

            $curline = '';
            $first = 1;
            
            foreach ($srv->channels as $chan)
            {
            	if (strlen($curline.', '.$chan->name) > 90) {
            	    if (!$first) {
            	    	ControlFactory::add_vgap($defs, -12);
            	    }
            	    ControlFactory::add_label($defs, null, $curline);
            	    $curline = $chan->name;
            	    $first = 0;
            	} else {
            	    if ($curline) {
            	    	$curline .= ', '.$chan->name;
            	    } else {
            	    	$curline = $chan->name;
            	    }
            	}
            }
            if (!$first) {
            	ControlFactory::add_vgap($defs, -12);
            }
            ControlFactory::add_label($defs, null, $curline);

            ControlFactory::add_close_dialog_button($defs,
                'Закрыть', 300, true);

            return ActionFactory::show_dialog("Состав пакета ТВ-каналов \"$srv->name\"",
                $defs, true);
        }

        return null;
    }

    public function get_all_folder_items(MediaURL $media_url, &$plugin_cookies)
    {
        $session = $this->plugin->get_session($plugin_cookies);
        $session->ensure_logged_in($plugin_cookies);
        $service_list = $session->getServiceList();
        try
        {
            $balance = -$session->getBalance();
        }
        catch (I24hException $e)
        {
            throw new DuneException(
                'Can not retrieve balance', 0,
                ActionFactory::show_error(
                    false,
                    'Ошибка проверки баланса',
                    array('Обратитесь в тех-поддержку.')));
        }

        $user_name = $plugin_cookies->user_name;

        $items = array();

        $details_common = "Текущий пользователь:|$user_name";
        $details_common .= "||Текущий баланс:|$balance руб";
        
        setlocale(LC_ALL, "ru_RU.UTF-8");
        
        foreach ($service_list as $srv)
        {
            if ($srv->id == 5) 
            	continue;
            $icon_url = $srv->isSubscribed ?
                'gui_skin://special_icons/included.aai' :
                'gui_skin://special_icons/excluded.aai';
            $icon_scale_factor = $srv->isSubscribed ? 1 : 1.5;
            // TODO Отоброжение тарифа
            $item_detailed_info = $details_common;
            $item_detailed_info .=
                "||Пакет:|$srv->name";
            $item_detailed_info .=
            	"||Тип:|". ($srv->tp == 1 ? "Основной" : "Дополнительный");
            if ($srv->active == 1)
            {
                $item_detailed_info .=
                    "||Выбранный тариф:|$srv->price руб. в мес";
            }

            $finish_time = '';
            $start_time = '';
            
            if ($srv->isSubscribed) {
            	
            	if (isset($srv->finished)) {
            		$d = date_create($srv->finished);
            		
            		$item_detailed_info .= "||Истекает:|".(strftime('%d.%m.%Y %H:%M', date_timestamp_get($d)));
            		$finish_time = strftime('%d.%m.%Y', date_timestamp_get($d));
            	}
            }
            
            if (isset($srv->starts)) {
            	$start_time = strftime('%d.%m.%Y', date_timestamp_get(date_create($srv->starts)));
            }
                
            $items[] = array
            (
                PluginRegularFolderItem::media_url => 
                    self::get_item_media_url_str($media_url->pin, 'service', $srv->id),
                PluginRegularFolderItem::caption => $srv->name . ($start_time ? ' (с '.$start_time.')' : ($finish_time ? ' (до '.$finish_time.')' : '')),
                PluginRegularFolderItem::view_item_params => array
                (
                    ViewItemParams::icon_path => $icon_url,
                    ViewItemParams::icon_scale_factor => $icon_scale_factor,
                    ViewItemParams::item_detailed_icon_path => 'gui_skin://large_icons/tv.aai',
                    ViewItemParams::item_detailed_info => $item_detailed_info,
                )
            );
        }
        
        $items[] = array
        (
            PluginRegularFolderItem::media_url => MediaURL::encode(
            	array (
            	    'screen_id'   => I24hSetupScreen::ID
            	    )),
            PluginRegularFolderItem::caption => '   [ Настройки... ]',
            PluginRegularFolderItem::view_item_params => array
            (
                ViewItemParams::icon_path => 'missing://',
                ViewItemParams::icon_scale_factor => 1,
                ViewItemParams::item_detailed_icon_path => 'gui_skin://large_icons/tv.aai',
                ViewItemParams::item_detailed_info => $details_common,
            )
        );

        /*$items[] = array
        (
            PluginRegularFolderItem::media_url => 
                self::get_item_media_url_str($media_url->pin, 'edit_pin', null),
            PluginRegularFolderItem::caption => '   [ Смена PIN-кода... ]',
            PluginRegularFolderItem::view_item_params => array
            (
                ViewItemParams::icon_path => 'missing://',
                ViewItemParams::icon_scale_factor => 1,
                ViewItemParams::item_detailed_icon_path => 'gui_skin://large_icons/tv.aai',
                ViewItemParams::item_detailed_info => $details_common,
            )
        );

        $items[] = array
        (
            PluginRegularFolderItem::media_url => 
                self::get_item_media_url_str($media_url->pin, 'reset_auth', null),
            PluginRegularFolderItem::caption => '   [ Сброс настроек авторизации ]',
            PluginRegularFolderItem::view_item_params => array
            (
                ViewItemParams::icon_path => 'missing://',
                ViewItemParams::icon_scale_factor => 1,
                ViewItemParams::item_detailed_icon_path => 'gui_skin://large_icons/tv.aai',
                ViewItemParams::item_detailed_info => $details_common,
            )
        );*/

        return $items;
    }

}

///////////////////////////////////////////////////////////////////////////
?>
