<?php

class I24H
{
    public static $SERVER = 'http://api.24h.tv/';
}

const I24H_ID_URL = 'v2/access?vendor_id=2&device_id=%s&stream=orig&_format=json';

const I24H_OTPS_URL = 'v2/otps';

const I24H_OTPS_AUTH_URL = 'v2/auth/otp';

const I24H_USER = 'v2/users';

const I24H_USER_DEVICE = 'v2/users/self/devices?access_token=%s';

const I24H_USER_SELF = 'v2/users/self?access_token=%s';

const I24H_SELF_NETWORK = 'v2/users/self/network?access_token=%s';

const I24H_USER_SELF_NETWORK = 'v2/users/self/network';

const I24H_USER_SUBSCRIPTION = 'v2/users/self/subscriptions/current?access_token=%s';

const I24H_LOGIN_URL =
    'v1/login?vendor_id=2&phone=%s&password=%s&device_id=%s&access-token=%s&_format=json';

const I24H_AUTH_DEVICE = 'v2/auth/device';


const I24H_DEVICE_LIST = 'v2/users/self/devices?access_token=%s';

const I24H_LOGOUT_URL = 'v1/logout?access-token=%s&_format=json';

const I24H_DELETE_DEVICE = 'v2/users/self/devices/%s?access_token=%s';

const I24H_GET_URL_URL =
    'http://ts://%s/dune/plugini.php?action=get_url&login=%s&pass=%s';
    
const I24H_SEND_PWD_URL =
    'v1/reset?vendor_id=2&phone=%s&device_id=%s&_format=json';

const I24H_SERVICES_URL = 'v2/packets?includes=channels,videos,availables&access_token=%s&format=json';
    //'v1/tariffs/search?access-token=%s&expand=channels&sort=priorety&_format=json';

const I24H_SERVICES_AV_URL = 'v2/users/self/subscriptions/current?access_token=%s&format=json';
    //'v1/payment/available?access-token=%s&_format=json';

const I24H_PAYSYSTEMS_URL =
    'v1/paysystem/search?access-token=%s&active=1&_format=json';

const I24H_INV_URL =
    'v1/profile/invoice?access-token=%s&_format=json';

const I24H_BALANCE_URL = 'v2/users/self?access_token=%s&format=json';
    //'v1/payment/cont?access-token=%s&_format=json';

const I24H_ADD_SERVICE = 'v2/users/self/subscriptions?access_token=%s&format=json';
   // 'v1/payment/create?access-token=%s&trf=%s&_format=json';

const I24H_DEL_SERVICE =
    'v1/payment/delete?access-token=%s&trf=%s&_format=json';

const I24H_PATCH_SERVICE = 'v2/users/self/subscriptions/%s?access_token=%s&format=json';

const I24H_GET_URL =
    'v1/url/index?access-token=%s&tm=0&id=%s';
    
const I24H_GET_TIME_URL =
    'v1/url/time?access-token=%s&tm=%s&id=%s';

const VIDEO_IN_PACKETS = 'v2/packets?includes=videos&access_token=%s&format=json';

const VIDEO_ALLOW = 'v2/users/self/subscriptions/current?includes=packet.videos&access_token=%s&format=json';

class I24hSession
{
    private $http_opts      = null;
    //private $logged_in = false;
    private $sid_name       = null;
    private $sid            = null;
    private $sid_uid        = null;
    private $token          = null;
    private $mbr_id         = null;
    private $mbr_phone      = null;
    private $parental_code  = null;
    private $ip_address     = null;
    private $support_phone  = null;
    private $support_email  = null;
    private $support_url    = null;
    private $provider_name  = null;
    private $provider_id    = null;
    private $terms          = null;
    private $proxy          = null;
    private $country        = null;
    private $city           = null;
    private $agreement      = null;
    private $provider_title = null;
    private $provider_desc  = null;
    private $has_tariffs    = null;
    private $current_packets        = null;
    private $channel_need_reloading = null;

    private $login_incorrect = false;

    private $account = null;
    private $services = null;
    private $settings = null;

    private $channel_list = null;
    private $unset_login_listener = null;
    private $service_list_cache = null;
    private $service_list = null;
    private $paysystems_list = null;
    private $videos_in_packets = null;
    private $current_videos = null;
    
    private $api_server = 'http://api.24h.tv/';

    public function __construct($unset_login_listener)
    {
        $this->http_opts = array (
            CURLOPT_FOLLOWLOCATION  => true,
            CURLOPT_MAXREDIRS       => 10,
// debug            CURLOPT_PROXY           => '192.168.1.9:3128',
            CURLOPT_COOKIEFILE      => '/tmp/24h2.cookies',
            CURLOPT_COOKIEJAR       => '/tmp/24h2.cookies'
        );

        $this->unset_login_listener = $unset_login_listener;
    }
    
    public function get_http_opts()
    { return $this->http_opts; }

    public function get_sid()
    { return $this->sid; }

    public function get_sid_uid()
    { return $this->sid_uid; }
    
    public function get_token()
    { return $this->token; }

    public function get_channel_list()
    { return $this->channel_list; }

    public function get_account()
    { return $this->account; }

    public function get_settings()
    { return $this->settings; }

    public function get_api_server()
    { return $this->api_server; }

    public function set_api_server($proxy)
    {
        $this->api_server = $proxy;
    }

    public function is_login_incorrect()
    { return $this->login_incorrect; }

    public function is_logged_in()
    { return ($this->mbr_id>0); }

    public function get_ip_address(){
        return $this->ip_address;
    }
    public function get_phone(){
        return $this->mbr_phone;
    }
    public function get_support_url(){
        return $this->support_url;
    }
    public function get_support_phone(){
        return $this->support_phone;
    }
    public function get_support_email(){
        return $this->support_email;
    }
    public function get_terms(){
        return $this->terms;
    }
    public function get_agreement(){
        return $this->agreement;
    }
    public function get_parental_code(){
        return $this->parental_code;
    }

    public function get_has_tariffs(){
        return $this->has_tariffs;
    }

    public function get_provider_name(){
        return $this->provider_name;
    }

    public function get_current_packets(){

        if (empty($this->current_packets)) {

            $subcribers = $this->api_call(
                sprintf(
                    $this->api_server . I24H_USER_SUBSCRIPTION, $this->get_token()), false);
            $this->has_tariffs = false;
            $this->current_packets = $subcribers;
            foreach ($subcribers as $s) {
                $this->has_tariffs = true;
            }
        }
        return $this->current_packets;
    }

    public function get_videos_in_packets()
    {
        if (empty($videos_in_packets)){
         //   hd_print("API get_videos_in_packets");
            $this->retriveVideosPackets();
        }
     //   hd_print(print_r($this->videos_in_packets,true));
        return $this->videos_in_packets;
    }

    public function get_current_videos()
    {
        if (empty($this->current_videos)){
       //     hd_print("API get_current_videos");
            $this->retriveCurrentVideos();
        }
   //     hd_print(print_r($this->current_videos,true));
        return $this->current_videos;
    }


    private function retriveVideosPackets()
    {
        $tarifs = $this->api_call(
            sprintf(
                $this->api_server . VIDEO_IN_PACKETS, $this->get_token()),false);

 //       hd_print("--------------------------------------");
//        hd_print(print_r($tarifs,true));
//        hd_print("--------------------------------------");
        unset($this->videos_in_packets);
        foreach($tarifs as $t) {
            if (!empty($t->videos)) {
                foreach ($t->videos as $v) {
                    if (!empty($v->id)) {
                        $this->videos_in_packets[$v->id][] = $t->name;
                    }
                }
            }
        }
    }

    private function retriveCurrentVideos()
    {
        $tariff = $this->api_call(
            sprintf(
                $this->api_server . VIDEO_ALLOW, $this->get_token()), false);
        unset($this->current_videos);
        foreach ($tariff as $t) {
            foreach ($t->packet->videos as $v) {
                $this->current_videos[$v->id] = $v->title;
            }
        }

        hd_print("retriveCurrentVideos: ".print_r($this->current_videos,true));

    }


    public function get_channel_need_reloading()
    {
        return $this->channel_need_reloading;
    }

    public function set_channel_need_reloading($flag){
        $this->channel_need_reloading = $flag;
    }

    public function get_movie_need_reloading()
    {
        return $this->movie_need_reloading;
    }

    public function set_movie_need_reloading($flag){
        $this->movie_need_reloading = $flag;
    }

    public function set_parental_code($new_code)
    {
        try
        {
            $this->http_opts[CURLOPT_CUSTOMREQUEST] ='PATCH';
            $this->http_opts[CURLOPT_HTTPHEADER]    ='Content-Type: application/json';

            //hd_print(print_r($this->http_opts,true));

            $doc = $this->api_call(
                sprintf(
                    $this->api_server . I24H_USER_SELF,$this->token),false,
                array("pincode"=>$new_code));
            $this->parental_code = $new_code;

            unset($this->http_opts[CURLOPT_CUSTOMREQUEST]);
            unset($this->http_opts[CURLOPT_HTTPHEADER]);

            return true;
        }
        catch (Exception $e)
        {
            hd_print("Error code = ".$e->getCode());
            return false;
        }

    }

    public function get_device_list(){
        try
        {

            $doc = $this->api_call(
                sprintf(
                    $this->api_server . I24H_DEVICE_LIST,$this->token),false);
            return $doc;
        }
        catch (Exception $e)
        {

            hd_print("Error code = ".$e->getCode());
            //$this->token
            //throw $this->dune_api_exception($e, 'Save pin code filed.', false);
            return false;

        }
    }

    public function unset_login()
    {
        $this->logged_in = false;
        $this->sid_name = null;
        $this->sid = null;

        $this->account = null;
        $this->services = null;

        $this->settings = null;

        $this->channel_list = null;
        $this->token = null;
        $this->mbr_id = null;

        unset($this->service_list);
        unset($this->service_list_cache);
        unset($this->videos_in_packets);
        unset($this->current_videos);
        unset($this->current_packets);
    }

    public function logout()
    {
    	if (isset($this->token)) {
    	    try
    	    {
                $this->delete_device($this->get_device_id());

                $this->http_opts[CURLOPT_HTTPHEADER]    = 'Content-Type: application/json';
    	        $this->http_opts[CURLOPT_CUSTOMREQUEST] = 'DELETE';
    	    	$doc = $this->api_call(sprintf($this->api_server . I24H_LOGOUT_URL, $this->token));
                unset($this->http_opts[CURLOPT_HTTPHEADER]);
                unset($this->http_opts[CURLOPT_CUSTOMREQUEST]);



    	    }
    	    catch (Exception $e)
    	    {
    	    	throw $this->dune_api_exception($e,
    	    	    'Stream url request failed.',
    	    	    true);
    	    }
    	    unset($this->token);
    	}

        unset($this->service_list);
        unset($this->service_list_cache);
        unset($this->videos_in_packets);
        unset($this->current_videos);
        unset($this->current_packets);
    	
        $this->unset_login_listener->unset_login();
    }

    ///////////////////////////////////////////////////////////////////////


    public function delete_device($device_id){
        if (isset($this->token)) {
            try
            {
                $this->http_opts[CURLOPT_HTTPHEADER]    = array('Content-Type: application/json');
                $this->http_opts[CURLOPT_CUSTOMREQUEST] = 'DELETE';
                $doc = $this->api_call(sprintf($this->api_server . I24H_DELETE_DEVICE, $device_id, $this->token));
                unset($this->http_opts[CURLOPT_HTTPHEADER]);
                unset($this->http_opts[CURLOPT_CUSTOMREQUEST]);
                return true;

            }
            catch (Exception $e)
            {
                throw $this->dune_api_exception($e,
                    'Stream url request failed.',
                    true);
            }
            unset($this->token);
        }else{
            return true;
        }
    }


    private function fetch_document($url)
    {
        $doc = HD::http_get_document($url, $this->http_opts);

        // assert($doc != null).

        return $doc;
    }

    private function fetch_post_document($url, $data)
    {
        $doc = HD::http_post_document($url, $data, $this->http_opts);

        // assert($doc != null).

        return $doc;
    }

    public function api_call($url, $silent=true, $data=null)
    {
        for (;;)
        {
            hd_print("api_call: $url");
            if (isset($data)) {
                hd_print("POST data: ".print_r($data,true));
            	$str = $this->fetch_post_document($url, $data);
            } else {
            	$str = $this->fetch_document($url);
            }

            if (!$silent)
                hd_print("Document:\n$str\n");

            $reply = json_decode($str);

            if (!$silent) hd_print("Reply from server:".print_r($reply,true));

            if ($reply === null)
                throw new Exception('Invalid data received from server');


            if (!empty($reply->status_code)&&($reply->status_code != 200)&&($reply->status_code != 400)&&($reply->status_code != 401)&&($reply->status_code != 403)) {

                hd_print("PRINT REPLY: ".print_r($reply,true));

                throw new Exception($reply->detail);
            }

            /*if ($reply->res != 'ok')
            {
                $ecode = 20; //intval($reply->error->code);

                if ($ecode === ECODE_QUERY_LIMIT_EXCEEDED)
                {
                    hd_print('API: query limit exceeded');
                    continue;
                }

                hd_print("API error: URL '$url' " .
                    "returned error with code=$ecode, msg=" .
                    $reply->message . '.');

                throw new I24hException(
                    $reply->message, $ecode);
            }*/

            break;
        }

        return $reply;
    }

    public function dune_api_exception($e, $def_caption, $is_playback)
    {
        $is_i24h_error = ($e instanceof I24hException);
        if (!$is_i24h_error)
        {
            hd_print('General exception: ' . $e->getMessage());
            $title = !$is_playback ? 'Error' :
                'Failed to connect to Internet.';
            $text_lines = $is_playback ? array() :
                array(
                 'Failed to connect to Internet.',
                 'Please check Internet connection.'
                );
            return new DuneException(
                $def_caption, -1,
                ActionFactory::show_error(false, $title, $text_lines));
        }
        hd_print('24h.TV API exception: ' .
            'code=' . $e->getCode() . ', message=' . $e->getMessage());

        $ecode = $e->getCode();
        $fatal = 
            $ecode == 11 || $ecode == 12 || $ecode == 13;

        if ($fatal)
            $this->logout();

        if ($ecode == 2)
        {
            $this->login_incorrect = true;

            $title = !$is_playback ? 'Ошибка' :
                'Неправильные данные подписки, проверьте номер и пароль.';
            $text_lines = $is_playback ? array() :
                array('Неправильные данные подписки, проверьте номер и пароль.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        if ($ecode == 4)
        {
            $title = !$is_playback ? 'Ошибка' :
                'Подписка истекла. Зайдите на 24h.tv и пополните счет для ее продления.';
            $text_lines = $is_playback ? array() :
                array('Подписка истекла. Зайдите на 24h.tv и пополните счет для ее продления.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        if ($ecode == 5)
        {
            $title = !$is_playback ? 'Ошибка' :
                'Неправильный телефонный номер. Используейте формат 79XXXXXXXXX (11 цифр).';
            $text_lines = $is_playback ? array() :
                array('Неправильный телефонный номер. Используейте формат 79XXXXXXXXX (11 цифр).');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }
        
        if ($ecode == 6)
        {
            $title = !$is_playback ? 'Ошибка' :
                'Слишком много запросов. Попробуйте попозже.';
            $text_lines = $is_playback ? array() :
                array('Слишком много запросов. Попробуйте попозже.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        if ($ecode == 20)
        {
            $title = !$is_playback ? 'Ошибка' :
                $e->getMessage();
            $text_lines = $is_playback ? array() :
                array($e->getMessage());
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

 /*        if ($ecode == 3)
        {
            $title = !$is_playback ? 'Error' :
                'Access denied for 10 minutes.';
            $text_lines = $is_playback ? array() :
                array('Access denied for 10 minutes.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        if ($ecode == 12)
        {
            $title = !$is_playback ? 'Error' :
                'Access denied. Probably subscription is used elsewhere.';
            $text_lines = $is_playback ? array() :
                array('Access denied.',
                    'Probably subscription is used elsewhere.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        if ($ecode == 11)
        {
            $title = !$is_playback ? 'Error' :
                'Subscription is used elsewhere.';
            $text_lines = $is_playback ? array() :
                array('Subscription is used elsewhere.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }*/

        if ($ecode == 19)
        {
            $title = !$is_playback ? 'Error' :
                'Current code is wrong.';
            $text_lines = $is_playback ? array() :
                array('Current code is wrong.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        $title = !$is_playback ? 'Ошибка' : $e->getMessage();
        $text_lines = $is_playback ? array() :
            array(rtrim($e->getMessage()),
                'Error code: ' . $ecode);
        return new DuneException(
            $def_caption, $ecode,
            ActionFactory::show_error($fatal, $title, $text_lines));
    }
    
    public function get_device_id() {
    	$h = fopen('/fconfig/dune_license.dlf', "r");
    	$l = fgets($h);
    	fclose($h);
    	
    	//hd_print($l);
    	preg_match('/\d+ \d+ (\S+) /', $l, $matches);
    	return preg_replace('/-/','',$matches[1]);
    }
    
    public function verify_device() {
    	$device_id = $this->get_device_id();
        $this->get_self_network();
    	
	hd_print("Trying to verify, device id ".$device_id);
    	
    	if ($device_id) { 
    	    $doc = "";
    	    try
    	    {
    	    	$doc = $this->api_call(
    	    	    sprintf(
    	    	    	$this->api_server . I24H_AUTH_DEVICE),false,
    	    	    	array('device_id'=>$device_id));//,
    	    	    //false);
    	    }
    	    catch (Exception $e)
    	    {

    	        hd_print("Error code = ".$e->getCode());

    	        hd_print("DOC: ".print_r($doc,true));

                //$this->token
    	    	//throw $this->dune_api_exception($e, 'Verify failed.', false);

    	    }

            if (!empty($doc->status_code)){
                //throw $this->dune_api_exception($e, 'Verify failed.', false);
                $this->token = "";

            }else{
                $this->token = $doc->access_token; /// ?????
//                $this->mbr_id = $doc->mbr_id;
//                $this->sid = $doc->stream->sid;
                $this->sid_name = 'sid';
//                $this->sid_uid = $doc->stream->uid;

                $user = $this->api_call(
                    sprintf(
                        $this->api_server . I24H_USER_SELF, $doc->access_token),false);

                $this->mbr_id = $user->id;
                $this->mbr_phone = $user->phone;
                $this->parental_code = $user->parental_code;
                $this->provider_name = $user->provider->name;

                hd_print("Verify ok, token=$this->token, mbr_id=$user->id, sid=$this->sid, uid=$this->sid_uid");



                $this->channel_need_reloading = true;
                $this->movie_need_reloading = true;

                hd_print("this->support_url ". $this->support_url);

                $this->get_current_packets();

            }

            unset($this->service_list);
            unset($this->service_list_cache);
            unset($this->videos_in_packets);
            unset($this->current_videos);
            unset($this->current_packets);

    	    ///TODO: NEED GET PROXY SETTINGS !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
/*    	    if ($doc->proxy) {
    	    	$this->api_server = $doc->proxy;
    	    	hd_print("Using proxy $this->api_server");
    	    	try
    	    	{
    	    		$doc = $this->api_call(
    	    			sprintf(
    	    				$this->api_server . I24H_ID_URL,
    	    				$device_id),
    	    			false);
    	    	}
    	    	catch (Exception $e)
    	    	{
    	    		throw $this->dune_api_exception($e, 'Verify failed.', false);
    	    	}
    	    }

    	    $this->token = $doc->token;
    	    $this->mbr_id = $doc->mbr_id;
    	    $this->sid = $doc->stream->sid;
    	    $this->sid_name = 'sid';
    	    $this->sid_uid = $doc->stream->uid;
    	    
    	    
    	    hd_print("Verify ok, token=$this->token, mbr_id=$this->mbr_id, sid=$this->sid, uid=$this->sid_uid");
*/


    	}
    }



    public function api_get_stream_url($playback_url)
    {
        $this->check_logged_in();

        if (substr($playback_url, 0, 12) === 'http://ts://')
            $playback_url = 'http://' . substr($playback_url, 12);
        else if (substr($playback_url, 0, 13) === 'http://mp4://')
            $playback_url = 'http://' . substr($playback_url, 13);

        try
        {
            $doc = $this->api_call($playback_url);
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Stream url request failed.',
                true);
        }

        if (!isset($doc->url))
            throw new Exception('Get_url failed.');

        $len = strlen($doc->url);
        $pos = $len;
        for ($i = 10; $i < $len; $i++)
        {
            if ($doc->url[$i] == '"' || $doc->url[$i] == ' ')
            {
                $pos = $i;
                break;
            }
        }

        $url = substr($doc->url, 0, $pos);

        if (substr($url, 0, 10) === 'http/ts://')
            return 'http://' . substr($url, 10);

        return $url;
    }

    private function api_login($user_name, $password)
    {
        $device_id = $this->get_device_id();

        hd_print("Trying to login, device id ".$device_id);
        try
        {

            $doc = $this->api_call(
                sprintf(
                    $this->api_server . I24H_OTPS_AUTH_URL), false,
                     array('phone' => preg_replace('/^8/', '7', $user_name), 'token' => $password));//, $device_id, $this->token),
                //false);

            hd_print("Response ".$doc->status_code." ".$doc->access_token);
            $this->token=$doc->access_token;

        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e, 'Ошибка авторизации!.', false);
        }
        if (!empty($doc->status_code)) {
            throw $this->dune_api_exception(new I24hException($doc->message, 12), $doc_message, false);
//            throw $this->dune_api_exception(new Exception($doc->message), $doc->message, false);
        }



        /*$this->logged_in = true;
        $this->sid = $doc->sid;
        $this->sid_name = $doc->sid_name;
        $this->login_incorrect = false;
        $this->account = $doc->account;
        $this->services = $doc->services;
        $this->settings = $doc->settings;*/
    }
/*
    private function api_login_old($user_name, $password)
    {
    	$device_id = $this->get_device_id();
    	
	hd_print("Trying to login, device id ".$device_id);
        try
        {
            
            $doc = $this->api_call(
                sprintf(
                    $this->api_server . I24H_LOGIN_URL,
                    preg_replace('/^8/', '7', $user_name), $password, $device_id, $this->token),
                false);
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e, 'Login failed.', false);
        }
        if ($doc->res == 'err') {
            throw $this->dune_api_exception(new I24hException($doc->message, 12), $doc_message, false);
//            throw $this->dune_api_exception(new Exception($doc->message), $doc->message, false);
        }

        
        
        /*$this->logged_in = true;
        $this->sid = $doc->sid;
        $this->sid_name = $doc->sid_name;
        $this->login_incorrect = false;
        $this->account = $doc->account;
        $this->services = $doc->services;
        $this->settings = $doc->settings;*/
//    }

    public function try_login(&$plugin_cookies)
    {
        if (!isset($plugin_cookies->user_name))
            throw new Exception('Не указан номер телефона!');

        try
        {
            $this->verify_device();
            $this->api_login(
                $plugin_cookies->user_name,
                $plugin_cookies->password);

           $this->verify_device();
           unset($this->service_list);
           unset($this->service_list_cache);
           unset($this->videos_in_packets);
           unset($this->current_videos);

           
           #$this->api_channel_list();
        }
        catch (Exception $e)
        {
            $this->logout();
            throw $e;
        }
    }

    public function ensure_logged_in(&$plugin_cookies)
    {
        if ($this->mbr_id)
            return;

        $this->verify_device();
        
        if (!$this->mbr_id) {
            throw $this->dune_api_exception(new I24hException('Вы не вошли в систему. Зайдите в настройки и укажите свой пароль.', 12), 12, false);
            //$this->try_login($plugin_cookies);
        }
    }

    public function check_logged_in()
    {
        if (!$this->mbr_id)
            throw new I24hException('Вы не вошли в систему. Зайдите в настройки и укажите свой пароль.', 12);
    }


    private function do_get_edit_subscription_defs(
        &$plugin_cookies, $handler, $add_params)
    {
        $defs = array();

        $user_name = isset($plugin_cookies->user_name) ?
            $plugin_cookies->user_name : '';
        $password = isset($plugin_cookies->password) ?
            $plugin_cookies->password : '';


        if (!empty($this->provider_id)) {

            ControlFactory::add_label($defs,
                null, 'Смотрите более 200 каналов с 24часаТВ! Удобный выбор из архива передач',
                array('indent' => 0));
            ControlFactory::add_vgap($defs, -20);
            ControlFactory::add_label($defs,
                null, 'и фильмов за 2 недели. Плюс доступ к видеотекам Amediateka и ЦТ-Премьера!',
                array('indent' => 0));


            ControlFactory::add_vgap($defs, 5);
            ControlFactory::add_label($defs,
                null, 'Для получения бесплатного доступа ко всем каналам на 14 дней введите номер',
                array('indent' => 0));
            ControlFactory::add_vgap($defs, -20);
            ControlFactory::add_label($defs,
                null, 'мобильного телефона и нажмите "Далее" для получения кода авторизации по SMS:',
                array('indent' => 0));


            ControlFactory::add_vgap($defs, 5);
            ControlFactory::add_label($defs,
                null, 'По истечении 14 дней вы можете бесплатно продолжить просмотр 20 основных каналов, ',
                array('indent' => 0));
            ControlFactory::add_vgap($defs, -20);
            ControlFactory::add_label($defs,
                null, 'либо оплатить услугу со счета своего интернет-провайдера - всего от 99 рублей в месяц.',
                array('indent' => 0));

        }else{

            $str_arr = preg_split('/;/',wordwrap($this->provider_desc,160,';',false));
            foreach ($str_arr as $s){
                ControlFactory::add_label($defs,
                    null, $s,
                    array('indent' => 0));
                ControlFactory::add_vgap($defs, -20);

            }

        }




        ControlFactory::add_vgap($defs, 20);

        ControlFactory::add_text_field($defs,
            $handler, $add_params,
            'user_name', 'Телефон:',
            /*$user_name*/'', true, false, false, false, 500);

        ControlFactory::add_vgap($defs, 20);

/*        ControlFactory::add_close_dialog_and_apply_button($defs,
            $handler, $add_params,
            'reg_show_agreement', 'Далее', 300);

*/
         ControlFactory::add_button($defs,
            $handler, $add_params,
            'reg_show_agreement', ' ', 'Далее', 500);

//        ControlFactory::add_button($defs,
//            $handler, $add_params,
//            'close', ' ', 'Отмена', 500);


////        ControlFactory::add_vgap($defs, 20);
//        ControlFactory::add_button($defs,
//            $handler, $add_params,
//            'get_sms', ' ', 'Получить код по SMS', 500);



//        ControlFactory::add_text_field($defs,
//            $handler, $add_params,
//            'password', 'Код авторизации:', /*$password*/'',
//            true, true, false, false, 500);
//
////        ControlFactory::add_vgap($defs, 20);
//
//        ControlFactory::add_button($defs,
//            $handler, $add_params,
//            'apply_subscription', '', 'Регистрация', 500);




//        ControlFactory::add_vgap($defs, 5);

        $defs[] = array
        (
            GuiControlDef::name => 'close',
            GuiControlDef::title => '',
            GuiControlDef::kind => GUI_CONTROL_BUTTON,
            GuiControlDef::specific_def =>
                array
                (
                    GuiButtonDef::caption => 'Отмена',
                    GuiButtonDef::width => 500,
                    GuiButtonDef::push_action =>
                        ActionFactory::close_dialog(),
                ),
            GuiControlDef::params => null,
        );

        ControlFactory::add_vgap($defs, 20);

        return $defs;
    }

/*
    private function do_get_edit_subscription_defs_old(
        &$plugin_cookies, $handler, $add_params)
    {
        $defs = array();

        $user_name = isset($plugin_cookies->user_name) ?
            $plugin_cookies->user_name : '';
        $password = isset($plugin_cookies->password) ?
            $plugin_cookies->password : '';

        ControlFactory::add_label($defs,
            null, 'Смотри 180 каналов с помощью 24h.tv! Удобный выбор из 30000 передач и фильмов',
            array('indent' => 0));
        ControlFactory::add_vgap($defs, -20);
        ControlFactory::add_label($defs,
            null, 'в архиве за 2 недели и Amediateka.',
            array('indent' => 0));
        ControlFactory::add_vgap($defs, 20);

        ControlFactory::add_text_field($defs,
            $handler, $add_params,
            'user_name', 'Телефон:',
            $user_name, true, false, false, false, 500);

        ControlFactory::add_text_field($defs,
            $handler, $add_params,
            'password', 'Пароль:', $password, 
            true, true, false, false, 500);

        ControlFactory::add_vgap($defs, 20);

        ControlFactory::add_button($defs,
            $handler, $add_params,
            'apply_subscription', null, 'Войти', 300);

        ControlFactory::add_vgap($defs, 5);
        ControlFactory::add_label($defs,
            null, 'Для получения бесплатного доступа ко всем каналам на 14 дней введите',
            array('indent' => 0));
        ControlFactory::add_vgap($defs, -20);
        ControlFactory::add_label($defs,
            null, 'номер мобильного и нажмите кнопку для получения пароля по SMS:',
            array('indent' => 0));

        ControlFactory::add_vgap($defs, -10);
        ControlFactory::add_button($defs,
            $handler, $add_params,
            'get_password', null, 'Получить пароль', 500);

        ControlFactory::add_vgap($defs, 5);
        ControlFactory::add_label($defs,
            null, 'По истечении 14 дней вы можете бесплатно продолжить просмотр 20 основных каналов, ',
            array('indent' => 0));
        ControlFactory::add_vgap($defs, -20);
        ControlFactory::add_label($defs,
            null, 'либо оплатить услугу со счета своего интернет-провайдера - всего от 99 рублей в месяц.',
            array('indent' => 0));
        
        ControlFactory::add_vgap($defs, 5);

        ControlFactory::add_close_dialog_button($defs,
            'Отмена', 300);

        return $defs;
    }
    */

    public function do_get_edit_subscription_action(
        &$plugin_cookies, $handler, $add_params = null)
    {
        return ActionFactory::show_dialog(
            empty($this->provider_title)?'Подписка на 24часаТВ':$this->provider_title,
            $this->do_get_edit_subscription_defs(
                $plugin_cookies, $handler, $add_params),
            true);
    }

    public function apply_subscription(&$plugin_cookies, &$user_input,$handler)
    {
        $post_action = null;
        if (!isset($user_input->control_id) ||
            $user_input->control_id != 'apply_subscription')
        {
            return false;
        }

        if ($user_input->user_name === '')
        {
            return array(
                'need_close_dialog' => false,
                'action' =>
                    ActionFactory::show_error(false,
                        'Error',
                        array('Не указан номер телефона или код авторизации.')));
        }


/*        $h = fopen('/tmp/run/versions.txt', "r");
        $fw = "";$pl = "";

        $file_handle = fopen("/tmp/run/versions.txt", "r");
        while (!feof($file_handle)) {
            $l = fgets($file_handle);
            if (preg_match('/firmware_version=(\S+)/', $l, $matches)){$fw = $matches[1];}
            if (preg_match('/product=(\S+)/', $l, $matches)){$pl = $matches[1];}

        }
        fclose($file_handle);
*/

//        while (!$h->eof()) {
//            // Echo one line from the file.
//            $l=$h->fgets();
//            if (preg_match('/firmware_version=(\S+)/', $l, $matches)){$fw = $matches[1];}
//            if (preg_match('/product=(\S+)/', $l, $matches)){$pl = $matches[1];}
//
//        }
////        $l = fgets($h);
//
//        fclose($h);
//        preg_match('/firmware_version=(\S+)/', $l, $matches);
//        $fw = $matches[0];
//        preg_match('/product=(\S+)/', $l, $matches);
//        $pl = $matches[0];

       // hd_print("fw=".$fw." pl=".$pl);

        // register device_id

        try
        {

            hd_print("OTPS AUTH ".$user_input->user_name." ".$user_input->password);
            $doc = $this->api_call(
                sprintf(
                    $this->api_server . I24H_OTPS_AUTH_URL), false,
                array('phone' => preg_replace('/^8/', '7', $user_input->user_name), 'token' => $user_input->password));//, $device_id, $this->token),
            //false);

//            hd_print("Response ".$doc->status_code." ".$doc->access_token);
//            $this->token=$doc->access_token;
            if (empty($doc->access_token)){

                $defs = array();

//                ControlFactory::add_vgap($defs, -20);
//                ControlFactory::add_label($defs,
//                    null, 'Вы ввели неверный код!',
//                    array('indent' => 0));
//                ControlFactory::add_vgap($defs, -20);
                ControlFactory::add_label($defs,
                    null, 'Введите код высланный по SMS,',
                    array('indent' => 0));
                ControlFactory::add_vgap($defs, -20);
                ControlFactory::add_label($defs,
                    null, 'На номер +'.$user_input->user_name,
                    array('indent' => 0));
                ControlFactory::add_vgap($defs, 30);
                ControlFactory::add_text_field($defs,$handler,null,'password','Код авторизации','',true,true,false,true,300);
                ControlFactory::add_vgap($defs, 30);
                ControlFactory::add_close_dialog_and_apply_button($defs,$handler,array(),'apply_subscription','Ввести код повторно',300);

                return array(
                    'need_close_dialog' => true,
                    'action' =>
                        ActionFactory::show_dialog('Неверный код, попробуйте снова!',$defs,true));

/*                return array(
                    'need_close_dialog' => false,
                    'action' =>
                        ActionFactory::show_error(false,
                            'Ошибка!',
                            array('Не верный код, попробуйте снова.')));
*/


            }else{
                $this->token = $doc->access_token;


                $post_action = ActionFactory::open_folder(MediaURL::encode(array('screen_id'=>I24hTopScreen::ID)),'24часаТВ');
                $post_action = ActionFactory::close_dialog_and_run($post_action);
                //reset_controls($this->do_get_edit_subscription_defs($plugin_cookies,  ,array()));




            }

        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e, 'Ошибка авторизации!.', false);
        }




       // I24H_USER_DEVICE

        //$plugin_cookies->user_id

        unset($this->http_opts[CURLOPT_HTTPHEADER]);
        unset($this->http_opts[CURLOPT_CUSTOMREQUEST]);

        $this->mbr_phone = $user_input->user_name;
        $plugin_cookies->user_name = $user_input->user_name;
        $plugin_cookies->password = $user_input->password;

        //$this->logout();


        //$doc->access_token

/*
        $doc = $this->api_call(sprintf($this->api_server . I24H_USER_DEVICE, $doc->access_token),false,array(
            "id"=> $this->get_device_id(),
            "vendor"=>"DUNE",
            "model"=>$pl,
            "version"=>$fw,
            "device_type"=>"stb",
            "serial"=>$this->get_device_id()
        ));


        if ($doc->status_code == 400 && $doc->detail[0] == 'Too many devices.') {

            hd_print("Go to I24hDeviceListScreen");
            //ActionFactory::open_folder(I24hDeviceListScreen::get_media_url_str());
            //ActionFactory::close_dialog();
            return array(
                'need_close_dialog' => true,
                'action' => ActionFactory::open_folder(I24hDeviceListScreen::get_media_url_str())
            );

          /*      ActionFactory::show_error(false,
                    'Error',
                    array('Превышен лимит подключенных устройств на одну учетную запись!.'))
          );




        }*/

        $reg_dev = $this->register_device($plugin_cookies);
        if ($reg_dev){return $reg_dev;}

        //$post_action = null;
            hd_print("Trying to login");
        try
        {
//            $this->try_login($plugin_cookies);
        }
        catch (DuneException $e)
        {
            $post_action = $e->get_error_action();

            $plugin_cookies->password = '';
        }


      //  hd_print("######################### ".print_r($post_action,true));


        return array(
            'need_close_dialog' => true,
            'action' => $post_action);
    }


    /**
     * @return null
     */
    public function register_device(&$plugin_cookies)
    {
        $h = fopen('/tmp/run/versions.txt', "r");
        $fw = "";$pl = "";

        $file_handle = fopen("/tmp/run/versions.txt", "r");
        while (!feof($file_handle)) {
            $l = fgets($file_handle);
            if (preg_match('/firmware_version=(\S+)/', $l, $matches)){$fw = $matches[1];}
            if (preg_match('/product=(\S+)/', $l, $matches)){$pl = $matches[1];}

        }
        fclose($file_handle);

        unset($this->http_opts[CURLOPT_HTTPHEADER]);
        unset($this->http_opts[CURLOPT_CUSTOMREQUEST]);


        //$this->logout();


        //$doc->access_token


        $doc = $this->api_call(sprintf($this->api_server . I24H_USER_DEVICE, $this->token),false,array(
            "id"=> $this->get_device_id(),
            "vendor"=>"DUNE",
            "model"=>empty($pl)?'dune':$pl,
            "version"=>empty($fw)?'unknown':$fw,
            "device_type"=>"stb",
            "serial"=>$this->get_device_id()
        ));


        if ((!empty($doc->status_code))&&($doc->status_code == 400 && $doc->detail[0] == 'Too many devices.')) {

            hd_print("Go to I24hDeviceListScreen");
            //ActionFactory::open_folder(I24hDeviceListScreen::get_media_url_str());
            //ActionFactory::close_dialog();

            $post_action = ActionFactory::open_folder(I24hDeviceListScreen::get_media_url_str(),"24часаТВ");
            $post_action = ActionFactory::close_dialog_and_run($post_action);
            

            return array(
                'need_close_dialog' => true,
                'action' => $post_action//ActionFactory::open_folder(I24hDeviceListScreen::get_media_url_str())
            );
        }else{
            return false;
        }

    }


    public function get_password(&$plugin_cookies, &$user_input, &$handler)
    {


       // hd_print("->get_password");

        if (!isset($user_input->control_id)){
           // hd_print("return false");
            return false;
        }

        if ($user_input->control_id == 'get_password' || $user_input->control_id == 'get_sms')
        {

        if ($user_input->user_name === '')
        {
            return array(
                'need_close_dialog' => false,
                'action' =>
                    ActionFactory::show_error(false,
                        'Запрос пароля',
                        array('Введите номер телефона для отправки пароля.')));
        }

//        try
//        {
            $user = $this->api_call($this->api_server . I24H_USER,false,array(
                    'username'  =>$user_input->user_name,
                    'first_name'=>$user_input->user_name,
                    'email'     =>$user_input->user_name.'@24h.tv',
                    'phone'     =>$user_input->user_name
            ));

            $plugin_cookies->user_name = $user_input->user_name;
            $plugin_cookies->password = $user_input->password;

            $this->mbr_phone = $user_input->user_name;

           // hd_print("I24H_USER: ".print_r($user,true));


            hd_print("user->status_code = ".$user->status_code);

            if (!empty($user)&&($user->status_code == 400)) {

            }elseif (!empty($user)&&($user->status_code == 403)){

                return array(
                    'need_close_dialog' => false,
                    'action' =>
                        ActionFactory::show_error(false,
                            'Ошибка!',
                            array('К сожалению, в вашем регионе работа приложения невозможна!','Страна: '.$this->country,'Город: '.$this->city,'IP: '.$this->ip_address)));


            }else {
                $plugin_cookies->user_id = empty($user->id) ? 0 : $user->id;
                $plugin_cookies->provider = empty($user->provider) ? 0 : $user->provider;
            }

            $doc = $this->api_call($this->api_server . I24H_OTPS_URL,false , array('phone'=>preg_replace('/^8/', '7', $user_input->user_name)));


            hd_print("doc: ".print_r($doc,true));

            if ($doc->status_code == 403) {

                return array(
                    'need_close_dialog' => true,
                    'action' =>
                        ActionFactory::show_error(false,
                            'Ошибка!',
                            array('К сожалению, в вашем регионе работа приложения невозможна!','Страна: '.$this->country,'Город: '.$this->city,'IP: '.$this->ip_address)));
            }


//        }
//        catch (Exception $e)
//        {
//            throw $this->dune_api_exception($e,
//                'Stream url request failed.',
//                true);
//        }
/*        if ($user_input->handler_id === 'entry' && $user_input->control_id === 'get_sms'){
            hd_print("Go window get password");
            return true;

        }
*/

            $defs = array();

            ControlFactory::add_label($defs,
                null, 'Введите код высланный по SMS,',
                array('indent' => 0));
            ControlFactory::add_vgap($defs, -20);
            ControlFactory::add_label($defs,
                null, 'На номер +'.$user_input->user_name,
                array('indent' => 0));
            ControlFactory::add_vgap($defs, 30);
            ControlFactory::add_text_field($defs,$handler,null,'password','Код авторизации','',true,true,false,true,300);
            ControlFactory::add_vgap($defs, 30);
            ControlFactory::add_close_dialog_and_apply_button($defs,$handler,array(),'apply_subscription','Применить',300);

            return array(
            'need_close_dialog' => true,
            'action' =>
                ActionFactory::show_dialog('Авторизация',$defs,true));

        }else if ($user_input->control_id == 'reg_show_agreement'){
            return $this->show_user_agreement($plugin_cookies, $user_input, $handler);
        }
        return false;

    }

   public function show_user_agreement(&$plugin_cookies, &$user_input, &$handler){

        if(!preg_match('/^7[0-9]{10}$/', $user_input->user_name))
        {
            return array(
                'need_close_dialog' => false,
                'action' =>
                    ActionFactory::show_error(false,
                        'Ошибка!',
                        array('Указан неверный номер телефона! Используйте формат 79XXXXXXXXX.')));
        }

        $this->get_self_network();

        $doc = HD::http_get_document($this->agreement);

        $defs = array();
        ControlFactory::add_multiline_label($defs, '', 'Управление: "P+" - перемотка вверх, "P-" - перемотка вниз.'."\n\n".$doc, 16);
        ControlFactory::add_close_dialog_button($defs, 'Назад', 300);


        ControlFactory::add_close_dialog_and_apply_button($defs,
            $handler, array(),
            'get_sms', 'Принять', 300);

        //ControlFactory::add_custom_button($defs,'user_agg','Принять','',300, );
//            ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'apply_agreement','Принять',300);
        //return ActionFactory::show_dialog('Пользовательское соглашение.', $defs, true, 1800);
        //hd_print("defs: ".print_r($defs,true));
        return array(
            'need_close_dialog' => false,
            'action' =>
                ActionFactory::show_dialog('Пользовательское соглашение.', $defs, true, 1800));


    }


    public function get_self_network()
    {
        $network = $this->api_call(
            sprintf(
                $this->api_server . I24H_USER_SELF_NETWORK),false);

        $this->ip_address    = $network->remote;
        $this->support_phone = $network->default->support->phone;
        $this->support_email = $network->default->support->email;
        $this->support_url   = $network->default->support->url;
        $this->terms         = $network->terms;
        $this->agreement     = $network->agreement;
        $this->provider_title= $network->default->login->title;
        $this->provider_desc = $network->default->login->description;
        $this->provider_id   = $network->provider->id;
        $this->provider_name = $network->provider->name;
        $this->country       = $network->country;
        $this->city          = $network->city;

//        $this->proxy         = $network->provider->proxy;
        if (!empty($network->provider->proxy)) $this->set_api_server($network->provider->proxy);

    }


    public function get_password_old(&$plugin_cookies, &$user_input)
    {
        if (!isset($user_input->control_id) ||
            $user_input->control_id != 'get_password')
        {
            return false;
        }

        if ($user_input->user_name === '')
        {
            return array(
                'need_close_dialog' => false,
                'action' =>
                    ActionFactory::show_error(false,
                        'Запрос пароля',
                        array('Введите номер телефона для отправки пароля.')));
        }

        try
        {
            $doc = $this->api_call(sprintf($this->api_server . I24H_SEND_PWD_URL, preg_replace('/^8/', '7', $user_input->user_name), $this->get_device_id()));
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Stream url request failed.',
                true);
        }

        return array(
                'need_close_dialog' => false,
                'action' =>
                ActionFactory::show_error(false,
                        'Запрос пароля',
                        array('На указанный номер отправлено SMS с паролем.',
                        	'Дождитесь сообщение, введите пароль и нажмите "Войти". '.$doc->message)));
        
        
        

    }

    public function retrieveURL($channel_id)
    {
        try
        {
            $doc = $this->api_call(sprintf($this->api_server . I24H_GET_URL, $this->token, $channel_id));
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Stream url request failed.',
                true);
        }

        return $doc;
    }
    
    public function retrieveTimeURL($tm, $channel_id)
    {
        try
        {
            $doc = $this->api_call(sprintf($this->api_server . I24H_GET_TIME_URL, $this->token, $tm, $channel_id));
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Stream url request failed.',
                true);
        }

        return $doc;
    }
    
    private function retrieveServiceList()
    {
        try
        {
            $doc = $this->api_call(sprintf($this->api_server . I24H_SERVICES_URL, $this->token),true);
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Stream url request failed.',
                true);
        }

        return $doc;
    }

    private function retrieveInvoicesList()
    {
        try
        {
            $doc = $this->api_call(sprintf($this->api_server . I24H_SERVICES_AV_URL, $this->token),true);
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Invoices url request failed.',
                true);
        }

        return $doc;
    }

    private function retrievePaysystemsList()
    {
        try
        {
            $doc = $this->api_call(sprintf($this->api_server . I24H_PAYSYSTEMS_URL, $this->token));
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Paysystems url request failed.',
                true);
        }

        return $doc;
    }

    private function updateInvoices()    
    {
        if (is_null($this->service_list_cache))
            $this->loadServiceList();
        
        $this->service_list = $this->service_list_cache;
        
        $invoices = $this->retrieveInvoicesList();
        $inv_arr = array();
        
        foreach ($invoices as $inv)
        {
            $inv_arr[$inv->id] = $inv;
        }
        
        foreach ($this->service_list as $srv)
        {
            if (isset($inv_arr[$srv->id])) {
                $srv->isSubscribed = 1;
				if (!($inv_arr[$srv->id]->repeated)) {
					$srv->finished = $inv_arr[$srv->id]->finish;
				}
				if ($inv_arr[$srv->id]->status == 7) {
					$srv->starts = $inv_arr[$srv->id]->start;
				}
            } else { 
            	$srv->isSubscribed = 0;
            }
        }
        
    }

    /*private function doAddServiceToSubscription($service_id)
    {
        $xml = $this->apiCall('addServiceToSubscription',
            array('serviceId' => $service_id));

        return $this->parseServiceList($xml);
    }

    private function doRemoveServiceFromSubscription($service_id)
    {
        $xml = $this->apiCall('removeServiceFromSubscription',
            array('serviceId' => $service_id));

        return $this->parseServiceList($xml);
    }*/
    
    private function loadServiceList()
    {
        $this->service_list_cache = $this->retrieveServiceList();

        if (is_null($this->service_list_cache))
            throw new Exception("Can not retrieve service list");
    }
    
    private function loadPaysystemsList()
    {
        $this->paysystems_list = $this->retrievePaysystemsList();
        
        if (is_null($this->paysystems_list))
            throw new Exception("Can not retrieve service list");
        
    }
    
    public function getServiceList()
    {
        if (is_null($this->service_list_cache))
            $this->loadServiceList();
        
        $this->updateInvoices();

        return $this->service_list;
    }

    public function getPaysystemsList()
    {
        if (is_null($this->paysystems_list))
            $this->loadPaysystemsList();
        
        return $this->paysystems_list;
    }

    public function getServiceById($service_id)
    {
        if (empty($this->service_list))
            $this->loadServiceList();

        $this->updateInvoices();

        foreach ($this->service_list as $srv)
        {
            if ($srv->id == $service_id)
                return $srv;
        }

        return null;
    }
    
    private function retrieveBalance()
    {
        try {
            $doc = $this->api_call(sprintf($this->api_server . I24H_BALANCE_URL, $this->token));
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Balance url request failed.',
                true);
        }
        //TODO проставить правильный баланс !!!
        return 0;//$doc->sum;
    }
    
    public function getBalance()
    {
        $balance = $this->retrieveBalance();

        if (is_null($balance))
            throw new Exception("Can not retrieve balance.");

        return $balance;
    }
    
    public function addServiceToSubscription($service_id)
    {
     //   $packet = array();
     //   $packet[] = array('packet_id'=>$service_id,'renew'=>'true');
        $this->http_opts[CURLOPT_HTTPHEADER]    = array('Content-Type: application/json');
        $packet = json_encode(array(array('packet_id'=>$service_id,'renew'=>'true')));


        try {
            $doc = $this->api_call(sprintf($this->api_server . I24H_ADD_SERVICE, $this->token),false,
                $packet
                );
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'request add service failed.',
                true);
        }

        unset($this->http_opts[CURLOPT_HTTPHEADER]);

        unset($this->service_list);
        unset($this->service_list_cache);
        unset($this->videos_in_packets);
        unset($this->current_videos);
        unset($this->current_packets);
        return $doc;


    }

    public function removeServiceFromSubscription($service_id)
    {
        try {
            $doc = $this->api_call(sprintf($this->api_server . I24H_DEL_SERVICE, $this->token, $service_id));
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'request add service failed.',
                true);
        }

        unset($this->service_list);
        unset($this->service_list_cache);
        unset($this->videos_in_packets);
        unset($this->current_videos);
        unset($this->current_packets);

        return $doc;
    }

    public function patchServiceToSubscription($service_id,$renew = false)
    {
        unset($this->current_packets);
        try
        {
            $this->http_opts[CURLOPT_CUSTOMREQUEST] ='PATCH';
            $this->http_opts[CURLOPT_HTTPHEADER]    ='Content-Type: application/json';

           // hd_print(print_r($this->http_opts,true));

            $doc = $this->api_call(
                sprintf(
                    $this->api_server . I24H_PATCH_SERVICE,$service_id,$this->token),false,
                array("renew"=> empty($renew)?"false":"true"));

            unset($this->http_opts[CURLOPT_CUSTOMREQUEST]);



            return true;
        }
        catch (Exception $e)
        {
            hd_print("Error code = ".$e->getCode());
            return false;
        }
    }


}
