<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/hashed_array.php';
require_once 'lib/tv/abstract_tv.php';
require_once 'lib/tv/default_epg_item.php';

require_once 'i24h_channel.php';

///////////////////////////////////////////////////////////////////////////

class I24hTv extends AbstractTv
{
    private $session;
    private $solo;
    private $products;
//    private $tariffs;

    public function __construct($session)
    {
        $this->session = $session;

        $f = fopen('/tmp/sysinfo.txt', 'r');
        $text=fread($f, 1024);
        fclose($f);
        $pos = strpos($text, 'solo4k');
        $posR11 = strpos($text,'_r11');

        if ($posR11 === false ) {
            if ($pos === false) {
                $this->solo = 'lower_cpu_load:1,';
            } else {
                $this->solo = '';
            }
        }else{
            $this->solo = '';
        }
		
		$this->products = array();
        
        parent::__construct(
            AbstractTv::MODE_CHANNELS_N_TO_M,
            I24hConfig::TV_FAVORITES_SUPPORTED,
            true);
    }

    public function get_fav_icon_url()
    {
        return I24hConfig::FAV_CHANNEL_GROUP_PANEL_ICON_PATH;
    }

    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////


    protected function load_channels(&$plugin_cookies)
    {


        $this->session->check_logged_in();

        $this->channels = new HashedArray();
        $this->groups = new HashedArray();

//        $this->tariffs = "sfsdvdvdsv";

        try {
            $doc = HD::http_get_document(
                sprintf($this->session->get_api_server() . I24hConfig::CHANNEL_IN_PACKET, $this->session->get_token()),
                $this->session->get_http_opts());
        } catch (Exception $e) {
            throw new DuneException(
                'HTTP network error', 0,
                ActionFactory::show_error(
                    true,
                    'Ошибка связи',
                    array(
                        'Не удаётся подключиться к IPTV-серверу.',
                        'Проверьте подключение к сети Интернет.')));
        }

        $xml = simplexml_load_string($doc);

        if ($xml === false) {
            hd_print("Error: can not parse XML document.");
            hd_print("XML-text: $doc.");
            throw new Exception('Illegal XML document');
        }

        if ($xml->getName() !== 'root') {
            hd_print("Error: unexpected node '" . $xml->getName() . "'. Expected: 'response'");
            throw new Exception('Invalid XML document');
        }

        $tariffs = array();
        foreach ($xml->children() as $packets) {
            foreach ($packets->channels->children() as $channels) {
                $tariffs[strval($channels->id)][] = strval($packets->name);
            }
        }

        if ($this->is_favorites_supported()) {
            $this->groups->put(
                new FavoritesGroup(
                    $this,
                    '__favorites',
                    I24hConfig::FAV_CHANNEL_GROUP_CAPTION,
                    I24hConfig::FAV_CHANNEL_GROUP_ICON_PATH,
                    I24hConfig::FAV_CHANNEL_GROUP_PANEL_ICON_PATH));
        }
        // Запросим из API список категорий с каналами доступных провайдеру
        try {
            $doc = HD::http_get_document(
                sprintf($this->session->get_api_server() . I24hConfig::CHANNEL_LIST_URL, $this->session->get_token()),
                $this->session->get_http_opts());
        } catch (Exception $e) {
            throw new DuneException(
                'HTTP network error', 0,
                ActionFactory::show_error(
                    true,
                    'Ошибка связи',
                    array(
                        'Не удаётся подключиться к IPTV-серверу.',
                        'Проверьте подключение к сети Интернет.')));
        }

        $xml = simplexml_load_string($doc);

        if ($xml === false) {
            hd_print("Error: can not parse XML document.");
            hd_print("XML-text: $doc.");
            throw new Exception('Illegal XML document');
        }

        if ($xml->getName() !== 'root') {
            hd_print("Error: unexpected node '" . $xml->getName() . "'. Expected: 'response'");
            throw new Exception('Invalid XML document');
        }

        $first = 0;
        $active_channels = array();
        $active_list = array();
        $passive_list = array();

        // Создаем категорию все каналы
        $this->groups->put(
            new AllChannelsGroup(
                $this,
                I24hConfig::ALL_CHANNEL_GROUP_CAPTION,
                I24hConfig::ALL_CHANNEL_GROUP_ICON_PATH,
                I24hConfig::ALL_CHANNEL_GROUP_PANEL_ICON_PATH));
        // Список категорий и каналов доступных провайдеру
        foreach ($xml->children() as $xml_tv_category) {
            // Создаем все категории доступных провайдеру
            $this->groups->put(
                new DefaultGroup(
                    strval($xml_tv_category->id),
                    strval($xml_tv_category->name),
                    strval('http://static.24h.tv/dune/channel/cat/cat' . $xml_tv_category->id . '.png?w=290&h=160'),
                    strval('http://static.24h.tv/dune/channel/cat/logo' . $xml_tv_category->id . '.png?w=290&h=160')
                ));
            // Пробежимся по всем каналам доступные провайдеру и заполним массив с данными по каналу
            foreach ($xml_tv_category->channels->children() as $ch) {
                if (empty($active_channels[strval($ch->id)])) {
                    $active_channels[strval($ch->id)] = array("id" => strval($ch->id), "name" => strval($ch->name), "icon" => strval($ch->icon), "groups" => array(strval($xml_tv_category->id)),"age"=>strval($ch->age));
                } else {
                    $arr = $active_channels[strval($ch->id)];
                    // Добавим канал в группу
                    $arr["groups"][] = strval($xml_tv_category->id);
                    $active_channels[strval($ch->id)] = $arr;
                }
            }
        }


        // Запросим из api список каналов доступных пользователю
        try {
            $doc = HD::http_get_document(
                sprintf($this->session->get_api_server() . I24hConfig::CHANNEL_USER_LIST, $this->session->get_token()),
                $this->session->get_http_opts());
        } catch (Exception $e) {
            throw new DuneException(
                'HTTP network error', 0,
                ActionFactory::show_error(
                    true,
                    'Ошибка связи',
                    array(
                        'Не удаётся подключиться к IPTV-серверу.',
                        'Проверьте подключение к сети Интернет.')));
        }

        $xml_user = simplexml_load_string($doc);

        if ($xml_user === false) {
            hd_print("Error: can not parse XML document.");
            hd_print("XML-text: $doc.");
            throw new Exception('Illegal XML document');
        }

        if ($xml_user->getName() !== 'root') {
            hd_print("Error: unexpected node '" . $xml_user->getName() . "'. Expected: 'response'");
            throw new Exception('Invalid XML document');
        }
        // пробежимся по каналам доступные пользователю
        foreach ($xml_user->children() as $users_channels) {
            if (isset($active_channels[strval($users_channels->id)])) {
                // Канал доступен пользователю, созраним его в хеше
                $active_list[strval($users_channels->id)] = 1;//$active_channels[strval($users_channels->id)];
            }
        }
        // Составим список закрытых каналов
        foreach ($active_channels as $c => $p) {
            // hd_print("passive_list channel id = ".print_r($active_channels[$c],true));
            $v = $active_channels[$c];
            if (empty($active_list[$v["id"]])) {
                // закрытый канал
                $passive_list[$v["id"]] = 1;
            }
        }

        foreach ($active_list as $ac => $n) {
            $v = $active_channels[$ac];
            // Создадим структуры каналов для Дюны

            $channel =
                new I24hChannel(
                    strval($v["id"]),
                    strval($v["name"]),
                    $v["icon"] . '?h=100',//'?w=245&h=140',
                    "",
                    intval($v["id"]),
                    14,
                    7,
                    true,
                    ($plugin_cookies->need_password_sextv=='yes')?(!empty($v["age"])&&$v["age"]==18?true:false):false,
                    $v["icon"] . '?h=100',//'?w=245&h=140'
                    $tariffs[$v["id"]]
                );

            foreach ($v["groups"] as $g) {
//                hd_print("active_list groups " . $v["id"] . " " . $g);

                $group = $this->groups->get($g);
                $channel->add_group($group);
            }
            foreach ($channel->get_groups() as $group) {
                if (isset($group)) {
                    $group->add_channel($channel);
                }
            }
            $this->channels->put($channel);

        }

        foreach ($passive_list as $ac => $n) {
            $v = $active_channels[$ac];
            // Создадим структуры каналов для Дюны
            $img = 'http://static.24h.tv/dune/channel/lock.png';//?w=245&h=140';//$v["icon"] . '?w=245&h=140';

            hd_print("!!!!! ".$v["id"]." ".implode(", ",$tariffs[$v["id"]]));


            $channel =
                new I24hChannel(
                    strval($v["id"]),
                    strval($v["name"]),
                    $img,
                    "passive",
                    intval($v["id"]),
                    14,
                    7,
                    true,
                    false,
                    $img,//$v["icon"] . '?h=100',//img //. '&lock=true'
                    $tariffs[$v["id"]]
                );

            foreach ($v["groups"] as $g) {
//                hd_print("passive_list groups " . $v["id"] . " " . $g);
                $group = $this->groups->get($g);
                $channel->add_group($group);
            }
            foreach ($channel->get_groups() as $group) {
                if (isset($group)) {
                    $group->add_channel($channel);
                }
            }


            $this->channels->put($channel);

        }
        // if (I24hConfig::CHANNEL_SORT_FUNC_CB != null)
        //     $this->channels->usort(I24hConfig::CHANNEL_SORT_FUNC_CB);
    }

    public function get_tv_info(MediaURL $media_url, &$plugin_cookies)
    {
        $this->session->ensure_logged_in($plugin_cookies);

        return parent::get_tv_info($media_url, &$plugin_cookies);
    }

    public function get_tv_stream_url($playback_url, &$plugin_cookies)
    {
        hd_print("***** get_tv_stream_url(".$playback_url.")");
        $url = $this->session->api_get_stream_url($playback_url);
        if (substr($url, 0, 1) === '/') {
        	$ss_bin_path = DuneSystem::$properties['install_dir_path'] . '/bin';
        	$cmd = $ss_bin_path . "/play.sh $ss_bin_path > /tmp/rrr.log";
        	$s = shell_exec($cmd);
        	if (!empty($s))
        		hd_print("$cmd - $s");
        	return 'http://ts://localhost:8081'.$url.'|||dune_params|||ll_prebuf_for_http_ts:1,'.$this->solo.'psi_opt:2,buffering_ms:300,ss_for_http_ts:1';
        } else {
        	return $url;
        }
    }
    
    public function save_history($product_id, &$plugin_cookies)
    {
    	if (isset($plugin_cookies->prodhist)) {
    		$prodhist = json_decode($plugin_cookies->prodhist, true);
    	} else {
    		$prodhist = array();
    	}
    	
    	foreach ($prodhist as $k => $val) {
    		if (($val['type'] == 1) && ($val['product_id'] == $product_id)) {
    			unset($prodhist[$k]);
    		}
    	}
    	
    	$len = array_unshift($prodhist, array(
    		'type' => 1,
    		'product_id' => $product_id));
    	
    	if ($len > 8) {
    		array_pop($prodhist);
    	}
    	$plugin_cookies->prodhist = json_encode($prodhist);
    	hd_print("prodhist: ".$plugin_cookies->prodhist);
    }


    public function get_tv_playback_url($channel_id, $archive_ts, $protect_code, &$plugin_cookies)
    {

        $channel = $this->get_channel($channel_id);
        $protect_code = empty($protect_code)?'':$protect_code;
        $restrict_ch = true;
        $need_req_pass = empty($plugin_cookies->need_password_sextv)?'no':$plugin_cookies->need_password_sextv;

        hd_print('plugin_cookies '.print_r($plugin_cookies,true));

        hd_print('TV secret code: need_req_pass = '.empty($need_req_pass)?'false':'true');

        if ($channel->is_protected() && $need_req_pass=='yes'){
            if ($protect_code === $this->session->get_parental_code()){
                $restrict_ch = true;
            }else{
                $restrict_ch = false;
            }
        }else{
            $restrict_ch = true;
        }

        if ($restrict_ch){
            hd_print("protect code is correct! ".$protect_code.' == '.$this->session->get_parental_code());
            $archive_ts = empty($archive_ts) ? 0 : $archive_ts;
            $archive_ts = ($archive_ts == -1) ? 0 : $archive_ts;
            $this->ensure_channels_loaded($plugin_cookies);
            $st_type = empty($plugin_cookies->stream_type) ? 'local' : $plugin_cookies->stream_type;

            hd_print("get url: " . $channel_id . " , " . $archive_ts . " , steam type " . $st_type . " , protect_code " . $protect_code);

            $ss_bin_path = DuneSystem::$properties['install_dir_path'] . '/bin';
            $cmd = $ss_bin_path . "/play.sh $ss_bin_path > /tmp/rrr.log";
            $s = shell_exec($cmd);

            // Запросим из API урл на вещание v2/channels/%s/stream?ts=%s&type=&s&access_token=%s
            try {
                $doc = HD::http_get_document(
                    sprintf($this->session->get_api_server() . I24hConfig::CHANNEL_URL, $channel_id, $archive_ts, $st_type, $this->session->get_token()),
                    $this->session->get_http_opts());
            } catch (Exception $e) {
                throw new DuneException(
                    'HTTP network error', 0,
                    ActionFactory::show_error(
                        true,
                        'Ошибка связи',
                        array(
                            'Не удаётся подключиться к IPTV-серверу.',
                            'Проверьте подключение к сети Интернет.')));
            }

            $xml = simplexml_load_string($doc);

            if ($xml === false) {
                hd_print("Error: can not parse XML document.");
                hd_print("XML-text: $doc.");
                throw new Exception('Illegal XML document');
            }

            if ($xml->getName() !== 'root') {
                hd_print("Error: unexpected node '" . $xml->getName() . "'. Expected: 'response'");
                throw new Exception('Invalid XML document');
            }
            $url = '';
            switch ($st_type) {
                case "hls":
                    $url = strval($xml->hls);
                    break;
                case "local":
                    $url = strval($xml->local);
                    break;
                case "local_low":
                    $url = strval($xml->local_low);
                    break;
                default:
                    throw new DuneException(
                        'HTTP network error', 0,
                        ActionFactory::show_error(
                            true,
                            'Ошибка связи',
                            array(
                                'Не удаётся подключиться к IPTV-серверу.',
                                'Проверьте подключение к сети Интернет.')));
            }
            //$url = 'http://localhost:18883/195.191.208.5-5054/743b65154244c700bca18bf82edc1572-f27a38371acf1bec6b23beecf1160eeb-0'; // udp
            //$url = 'http://localhost:18883/195.191.208.5-5054/8d59e7e8ab42e07c9deea255554e5e99-f27a38371acf1bec6b23beecf1160eeb-1500'; // udp low
            //$url = 'http://stream7.24h.tv:8090/hlsin/0adc52c24570ee7fa6bf6d75687e38e1-f27a38371acf1bec6b23beecf1160eeb/index.m3u8'; // hls

            $channel = $this->get_channel($channel_id);
            if ($channel->get_streaming_url() != "passive") {

                if ($archive_ts > 0) {
                    if (isset($this->products[$channel_id][intval($archive_ts)])) {
                        $pid = $this->products[$channel_id][intval($archive_ts)];
                        hd_print("Saving product id: $pid");

                        if (!$channel->is_protected()) $this->save_history($pid, $plugin_cookies);

                    }
                    hd_print("GET ARCHIVE STREAM URL archive_ts = " . $archive_ts . " url =  " . $url);
                    return preg_replace('/^http:\/\/localhost:/', 'http://ts://localhost:', $url) .
                        '|||dune_params|||ll_prebuf_for_http_ts:1,' . $this->solo . 'psi_opt:2,buffering_ms:300,ss_for_http_ts:1,keep_pic:0';
                } else {
                    hd_print("1111 GET STREAM URL " . $url);
                    return preg_replace('/^http:\/\/localhost:/', 'http://ts://localhost:', $url) .
                        '|||dune_params|||ll_prebuf_for_http_ts:1,' . $this->solo . 'psi_opt:2,buffering_ms:300,ss_for_http_ts:1,keep_pic:0';
                }
            }else{
                hd_print("PASSIVE CHANNEL!!!!!");
                $def = array();

                //  return  ActionFactory::show_dialog('Недоступный канал',$def);// show_error(
                       // false,
                       // 'Недоступный канал!',
                       // array(
                        //    'Не удаётся подключиться к IPTV-серверу.',
                         ///   'Проверьте подключение к сети Интернет.'))
              //  return 'Канал Вам не достпен!';
              return 'http://ts://195.191.208.3:18987/238.254.0.2-1234|||dune_params|||ll_prebuf_for_http_ts:1,'.$this->solo.'psi_opt:2,buffering_ms:300,ss_for_http_ts:1,keep_pic:0';
            }
            return;
        }else{
            hd_print("protect code is incorrect! ".$protect_code.' != 0000 !!!!!!');
            hd_print("-----------");
            $channel = $this->get_channel($channel_id);
            if ($channel->get_streaming_url() == "passive") {
                hd_print("PASSIVE CHANNEL!!!!!");


//                    ActionFactory::show_error(
//                        true,
//                        'Канал Вам не достпен!');
                //    return '';
                return 'http://ts://195.191.208.3:18987/238.254.0.2-1234|||dune_params|||ll_prebuf_for_http_ts:1,'.$this->solo.'psi_opt:2,buffering_ms:300,ss_for_http_ts:1,keep_pic:0';
            }else {
                return "protected";
            }
            return "protected";
        }

    }

    public function get_tv_playback_url_old($channel_id, $archive_ts, $protect_code, &$plugin_cookies)
    {
        // TODO Вставляем урл из апи тут
        $this->ensure_channels_loaded($plugin_cookies);
        $st_type = empty($plugin_cookies->stream_type)?'local':$plugin_cookies->stream_type;

        hd_print("get url: ". $channel_id." , ".$archive_ts." , steam type ".$st_type." , protect_code ".$protect_code);









        /*if (substr($url, 0, 4) === 'http') {
            if ($archive_ts > 0) {
            	$url = sprintf($this->get_channel($channel_id)->get_archive_url(), $archive_ts);
            } else {
            	$url = $this->get_channel($channel_id)->get_streaming_url();
            }
            
            return $url;
            
        } else*/ {
            
            if ($archive_ts > 0) {
            	if (isset($this->products[$channel_id][intval($archive_ts)])) {
            			$pid = $this->products[$channel_id][intval($archive_ts)];
            			hd_print("Saving product id: $pid");
            			$this->save_history($pid, $plugin_cookies);
            	}
                // STEAM URL
            	$url = sprintf($this->get_channel($channel_id)->get_archive_url(), $this->session->get_sid(), $archive_ts);
            	//
                if (substr($url, 0, 1) === '/') {
                    $ss_bin_path = DuneSystem::$properties['install_dir_path'] . '/bin';
                    $cmd = $ss_bin_path . "/play.sh $ss_bin_path > /tmp/rrr.log";
                    $s = shell_exec($cmd);
                    if (!empty($s))
                    	hd_print("$cmd - $s");
                    return 'http://ts://localhost:8081'.$url.'|||dune_params|||ll_prebuf_for_http_ts:1,'.$this->solo.'psi_opt:2,buffering_ms:300,ss_for_http_ts:1,keep_pic:0';
                } else if (substr($url, 0, 12) === 'http://ts://') {
                    $url = sprintf($this->get_channel($channel_id)->get_archive_url(), $archive_ts);
                    return $url;
                } else {
                	$doc = $this->session->retrieveTimeURL($archive_ts, $channel_id);
                	return $doc->hls;
                    //$url = sprintf($this->get_channel($channel_id)->get_archive_url(), $archive_ts);
                    //return $url;
                    //return $url."|||dune_params|||,ss_for_http_ts:1";
                }
            } else {

                $url = sprintf($this->get_channel($channel_id)->get_streaming_url(), $this->session->get_sid());


                //$url1 = 'http://localhost:18883/195.191.208.5-5054/743b65154244c700bca18bf82edc1572-f27a38371acf1bec6b23beecf1160eeb-0'; // udp
                $url1 = 'http://localhost:18883/195.191.208.5-5054/8d59e7e8ab42e07c9deea255554e5e99-f27a38371acf1bec6b23beecf1160eeb-1500'; // udp low
                //$url1 = 'http://stream7.24h.tv:8090/hlsin/0adc52c24570ee7fa6bf6d75687e38e1-f27a38371acf1bec6b23beecf1160eeb/index.m3u8'; // hls

                $url = preg_replace('/^http:\/\/localhost:/','http://ts://localhost:',$url1);
                hd_print("GET STREAM URL ".$url);

                if (substr($url, 0, 1) === '/') {
                    $ss_bin_path = DuneSystem::$properties['install_dir_path'] . '/bin';
                    $cmd = $ss_bin_path . "/play.sh $ss_bin_path > /tmp/rrr.log";
                    $s = shell_exec($cmd);
                    if (!empty($s))
                    	hd_print("$cmd - $s");

                    return //'http://ts://localhost:8081/'.
                        $url.'|||dune_params|||ll_prebuf_for_http_ts:1,'.$this->solo.'psi_opt:2,buffering_ms:300,ss_for_http_ts:1,keep_pic:0';
                } else if (substr($url, 0, 12) === 'http://ts://') {
                    $url = $this->get_channel($channel_id)->get_streaming_url();
                    return $url;
                } else {
                	$doc = $this->session->retrieveURL($channel_id);
                	return $doc->hls;
                    //$url = $this->get_channel($channel_id)->get_streaming_url();
                    //return $url;
                    //return $url."|||dune_params|||,ss_for_http_ts:1";
                }
            }
        }

        return;                                                                                                                                                          

        /*

        $this->ensure_channels_loaded($plugin_cookies);

        echo "get url: $channel_id, $archive_ts\n";

        $url = sprintf(I24H_GET_URL_URL, I24H::$SERVER,
        		$plugin_cookies->user_name,
        		$plugin_cookies->password
        	);
        
        $url .= "&cid=$channel_id";

        if (intval($archive_ts) > 0)
            $url .= "&start_ts=$archive_ts";
        if (isset($protect_code) && $protect_code !== '')
            $url .= "&protect_code=$protect_code";

        return $url;*/
    }
    
    ///////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////

    public function get_day_epg_iterator($channel_id, $day_start_ts, &$plugin_cookies)
    {
        if (I24hConfig::EPG_URL_FORMAT == null)
            return array();

        try
        {
            $doc =
                HD::http_get_document(
                    sprintf(
                        $this->session->get_api_server() . I24hConfig::EPG_URL_FORMAT,

                        $channel_id,
                        $day_start_ts,
                        $day_start_ts+24*3600,
                        $this->session->get_token()
                        ));
        }
        catch (Exception $e)
        {
            throw new DuneException(
                'HTTP network error', 0,
                ActionFactory::show_error(
                    true,
                    'Ошибка связи',
                    array(
                        'Не удаётся подключиться к IPTV-серверу.',
                        'Проверьте подключение к сети Интернет.')));
        }
     
        $xml = simplexml_load_string($doc);

        if ($xml === false)
        {
            hd_print("Error: can not parse XML document.");
            hd_print( "XML-text: $doc.");
            throw new Exception('Illegal XML document');
        }

        if ($xml->getName() !== 'root')
        {
            hd_print("Error: unexpected node '" . $xml->getName() . "'. Expected: 'NewDataSet'");
            throw new Exception('Invalid XML document');
        }

        $epg = array();

        foreach ($xml->children() as $epg_item)
        {
//            if ($epg_item->getName() !== 'item')
//            {
//                hd_print("Error: unexpected node '" . $epg_item->getName() .
//                    "'. Expected: 'item'");
//                throw new Exception('Invalid XML document');
//            }

            $epg[] =
                new DefaultEpgItem(
                    strval($epg_item->program->title),
                    strval($epg_item->program->description),
                    intval($epg_item->timestamp),
                    intval($epg_item->timestamp)+intval($epg_item->duration));
                
            if (!isset($this->products[$channel_id])) {
            	$this->products[$channel_id] = array();
            }
            $this->products[$channel_id][intval($epg_item->timestamp)] = intval($epg_item->id);//product_id);
        }

        return
            new EpgIterator(
                $epg,
                $day_start_ts,
                $day_start_ts + 86400);
    }

    public function folder_entered(MediaURL $media_url, &$plugin_cookies)
    {
        if (!isset($media_url->screen_id) ||
            $media_url->screen_id === TvGroupListScreen::ID)
        {
           // $this->session->logout();
        }

        $this->session->ensure_logged_in($plugin_cookies);
    }
    
    public function have_channel($channel_id)
    {
        $c = $this->channels->get($channel_id);

        if (is_null($c))
            return false;

        return true;
    }

}

///////////////////////////////////////////////////////////////////////////
?>
