<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/screen.php';
require_once 'lib/vod/vod.php';
require_once 'lib/vod/vod_series_list_screen.php';

class VodMovieScreen implements Screen, UserInputHandler
{
    const ID = 'vod_movie';

    public static function get_media_url_str($movie_id)
    {
    	//hd_print(" !! $movie_id");
        return MediaURL::encode(
            array(
                'screen_id' => self::ID,
                'movie_id' => $movie_id));
    }

    ///////////////////////////////////////////////////////////////////////

    private $vod;

    public function __construct(Vod $vod)
    {
        $this->vod = $vod;

        UserInputHandlerRegistry::get_instance()->
            register_handler($this);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_id()
    { return self::ID; }

    public function get_handler_id()
    { return self::ID; }

    ///////////////////////////////////////////////////////////////////////

    public function get_folder_view(MediaURL $media_url, &$plugin_cookies)
    {
        $this->vod->folder_entered($media_url, $plugin_cookies);

        $movie = $this->vod->get_loaded_movie($media_url->movie_id, $plugin_cookies);
        if ($movie === null)
        {
            // TODO: dialog?
            return null;
        }

        $left_button_caption = $this->vod->get_movie_left_button_caption(
            $media_url, $plugin_cookies);
      //  hd_print("series url: ".$movie->series_list[0]->playback_url);


        //$pl = MediaURL::decode($movie->series_list[0]->playback_url);

        /*$left_button_action = (count($movie->series_list) > 1) ?
        	$this->vod->get_movie_left_button_action($media_url, $plugin_cookies) :

            array
        	(
        	    GuiAction::handler_string_id => PLUGIN_TV_PLAY_ACTION_ID,
        	    GuiAction::data =>
        	    array
        	    (
        	    	PluginOpenFolderActionData::media_url => $movie->series_list[0]->playback_url
        	    	)
        	    );


        */
        $left_button_action =
        	$this->vod->get_movie_left_button_action($media_url, $plugin_cookies);
        
        //$this->vod->get_movie_left_button_action(
        //    $media_url, $plugin_cookies);

        $has_right_button = $this->vod->is_favorites_supported();
        $right_button_caption = null;
        $right_button_action = null;
        if ($has_right_button)
        {
            $this->vod->ensure_favorites_loaded($plugin_cookies);

            $is_favorite = $this->vod->is_favorite_movie_id($plugin_cookies,$movie->id);
            $right_button_caption = $is_favorite ?
                'Удалить из избранного' : 'Добавить в избранное';
            $right_button_action = UserInputHandlerRegistry::create_action(
                $this, 'favorites',
                array('movie_id' => $movie->id));
        }

        $movie_folder_view = array
        (
            PluginMovieFolderView::movie => $movie->get_movie_array(),
            PluginMovieFolderView::left_button_caption => $left_button_caption,
            PluginMovieFolderView::left_button_action => $left_button_action,
            PluginMovieFolderView::has_right_button => $has_right_button,
            PluginMovieFolderView::right_button_caption => $right_button_caption,
            PluginMovieFolderView::right_button_action => $right_button_action,
            PluginMovieFolderView::params => array
            (
                    PluginFolderViewParams::background_url => I24hConfig::BG_PIC,  // 24h.tv
            ),
            PluginMovieFolderView::has_multiple_series =>
                true,//(count($movie->series_list) > 1),
            PluginMovieFolderView::series_media_url =>
                VodSeriesListScreen::get_media_url_str($movie->id),
            
        );

        return array
        (
            PluginFolderView::multiple_views_supported  => false,
            PluginFolderView::archive                   => null,
            PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_MOVIE,
            PluginFolderView::data                      => $movie_folder_view,
        );
    }

    ///////////////////////////////////////////////////////////////////////

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
      //  hd_print('Movie: handle_user_input:');
     //  foreach ($user_input as $key => $value)
      //      hd_print("  $key => $value");

        if ($user_input->control_id == 'favorites')
        {
            $movie_id = $user_input->movie_id;

            $is_favorite = $this->vod->is_favorite_movie_id($plugin_cookies,$movie_id);
            if ($is_favorite)
                $this->vod->remove_favorite_movie($movie_id, $plugin_cookies);
            else
                $this->vod->add_favorite_movie($movie_id, $plugin_cookies);

            $message = $is_favorite ?
                'Фильм удален из избранного' :
                'Фильм добавлен в избранное';

            return ActionFactory::show_title_dialog($message,
                ActionFactory::invalidate_folders(
                    array(
                        self::get_media_url_str($movie_id),
                        VodFavoritesScreen::get_media_url_str(),
                    )));
        }

        return null;
    }
}

///////////////////////////////////////////////////////////////////////////
?>
